@@ -63,14 +63,11 @@
 #' @param delta_n A numeric scalar.  The amount by which n is increased
 #'   (or decreased) after one click of the + (or -) button in the parameter
 #'   window.
-#' @param arrow  A logical scalar.  Should an arrow be included to show the
+#' @param arrow A logical scalar.  Should an arrow be included to show the
 #'   simulated sample maximum from the top plot being placed into the
 #'   bottom plot?
-#' @param pos A numeric integer.  Used in calls to \code{\link{assign}}
-#'   to make information available across successive frames of a movie.
-#'   By default, uses the current environment.
-#' @param envir An alternative way (to \code{pos}) of specifying the
-#'   environment. See \code{\link{environment}}.
+#' @param leg_cex The argument \code{cex} to \code{\link[graphics]{legend}}.
+#'   Allows the size of the legend to be controlled manually.
 #' @param ... Additional arguments to the rpanel functions
 #'   \code{\link[rpanel]{rp.button}} and
 #'   \code{\link[rpanel]{rp.doublebutton}}, not including \code{panel},
@@ -93,9 +90,12 @@
 #'   Samples of size \code{n} are repeatedly simulated from the distribution
 #'   chosen using \code{distn}.  These samples are summarized using a plot
 #'   that appears at the top of the movie screen.  For each sample the mean
-#'   of these \code{n} values is calculated, stored and added to another plot.
+#'   of these \code{n} values is calculated, stored and added to another plot,
+#'   situated below the first plot.
 #'   This plot is either a histogram or an empirical c.d.f., chosen using a
 #'   radio button.
+#'   A \code{\link[graphics]{rug}} is added to a histogram provided that it
+#'   contains no more than 1000 points.
 #'
 #'   The p.d.f. (for a continuous variable) or p.m.f. (for a discrete variable)
 #'   of the original variables is added to the top plot.
@@ -131,7 +131,12 @@
 #' @export
 clt <- function(n = 20, distn, params = list(), panel_plot = TRUE, hscale = NA,
                 vscale = hscale, n_add = 1, delta_n = 1, arrow = TRUE,
-                pos = 1, envir = as.environment(pos), ...) {
+                leg_cex = 1.25, ...) {
+  if (!tcltk::is.tclObj(tcltk::tclRequire("BWidget"))) {
+    message("Package BWidget was not found.")
+    message("Please see the smovie README file for information.")
+    return()
+  }
   temp <- set_scales(hscale, vscale)
   hscale <- temp$hscale
   vscale <- temp$vscale
@@ -301,10 +306,11 @@ clt <- function(n = 20, distn, params = list(), panel_plot = TRUE, hscale = NA,
   leg_pos <- set_leg_pos(distn, fun_args)
   top_leg_pos <- leg_pos$top_leg_pos
   bottom_leg_pos <- leg_pos$bottom_leg_pos
-  # Assign variables to an environment so that they can be accessed inside
-  # clt_exponential_movie_plot()
+  # Set a unique panel name to enable saving of objects to the correct panel
+  now_time <- strsplit(substr(date(), 12, 19), ":")[[1]]
+  now_time <- paste(now_time[1], now_time[2], now_time[3], sep = "")
+  my_panelname <- paste("clt_", now_time, sep = "")
   old_n <- 0
-  assign("old_n", old_n, envir = envir)
   # Store the mean and standard deviation of the underlying distribution
   if (distn == "hypergeometric") {
     hp <- fun_args$m / (fun_args$m + fun_args$n)
@@ -392,27 +398,43 @@ clt <- function(n = 20, distn, params = list(), panel_plot = TRUE, hscale = NA,
                        gamma(1 + 1 / fun_args$shape) ^ 2),
                      "exponential" = 1 / fun_args$rate)
   # Create buttons for movie
-  pdf_or_cdf <- "pdf"
   show_dens <- FALSE
-  assign("old_pdf_or_cdf", pdf_or_cdf, envir = envir)
-  assign("old_show_dens", show_dens, envir = envir)
-  cltpanel <- rpanel::rp.control("central limit theorem", n = n, n_add = n_add,
+  pdf_or_cdf <- "pdf"
+  old_y <- NULL
+  save_last_y <- NULL
+  sample_means <- NULL
+  cltpanel <- rpanel::rp.control("central limit theorem",
+                                 panelname = my_panelname, n = n, n_add = n_add,
                                  dfun = dfun, qfun = qfun, rfun = rfun,
                                  pfun = pfun, fun_args = fun_args,
                                  distn = distn, top_range = top_range,
                                  top_p_vec = top_p_vec,
                                  bottom_p_vec = bottom_p_vec,
-                                 pdf_or_cdf = "pdf", show_dens = show_dens,
+                                 pdf_or_cdf = pdf_or_cdf, show_dens = show_dens,
                                  top_leg_pos = top_leg_pos,
                                  bottom_leg_pos = bottom_leg_pos,
                                  xlab = xlab, arrow = arrow,
                                  distn_mean = distn_mean, distn_sd = distn_sd,
                                  discrete_distn = discrete_distn,
-                                 envir = envir)
+                                 old_n = old_n, old_pdf_or_cdf = pdf_or_cdf,
+                                 old_show_dens = show_dens,
+                                 old_y = old_y, save_last_y = save_last_y,
+                                 sample_means = sample_means,
+                                 leg_cex = leg_cex)
   #
   redraw_plot <- NULL
   panel_redraw <- function(panel) {
     rpanel::rp.tkrreplot(panel = panel, name = redraw_plot)
+    # rp.tkrreplot() doesn't update the panel automatically, so do it manually
+    # Get ...
+    panel$sample_means <- rpanel::rp.var.get(my_panelname, "sample_means")
+    panel$old_n <- rpanel::rp.var.get(my_panelname, "old_n")
+    panel$old_pdf_or_cdf <- rpanel::rp.var.get(my_panelname, "old_pdf_or_cdf")
+    panel$old_show_dens <- rpanel::rp.var.get(my_panelname, "old_show_dens")
+    panel$old_y <- rpanel::rp.var.get(my_panelname, "old_y")
+    panel$save_last_y <- rpanel::rp.var.get(my_panelname, "save_last_y")
+    # Put ...
+    rpanel::rp.control.put(my_panelname, panel)
     return(panel)
   }
   if (panel_plot & !requireNamespace("tkrplot", quietly = TRUE)) {
@@ -420,15 +442,10 @@ clt <- function(n = 20, distn, params = list(), panel_plot = TRUE, hscale = NA,
     panel_plot <- FALSE
   }
   if (panel_plot) {
-    # Set a seed and then reset it before the panel is redrawn so that only
-    # one arrow and sample mean appears in the first plot
-    my_seed <- round(stats::runif(1, 0, 1000))
-    set.seed(my_seed)
     rpanel::rp.tkrplot(panel = cltpanel, name = redraw_plot,
                        plotfun = cltmovie_plot, pos = "right",
                        hscale = hscale, vscale = vscale, background = "white")
     action <- panel_redraw
-    set.seed(my_seed)
   } else {
     action <- cltmovie_plot
   }
@@ -456,22 +473,29 @@ clt <- function(n = 20, distn, params = list(), panel_plot = TRUE, hscale = NA,
                         title = "pdf or cdf in bottom plot", action = action)
   rpanel::rp.checkbox(panel = cltpanel, show_dens,
                       labels = "show normal pdf/cdf", action = action)
-  rpanel::rp.do(panel = cltpanel, action = action)
+  if (!panel_plot) {
+    rpanel::rp.do(panel = cltpanel, action = action)
+  }
   return(invisible())
 }
 
 # Function to be called by ett().
 
 cltmovie_plot <- function(panel) {
-  with(panel, {
-    old_par <- graphics::par(no.readonly = TRUE)
+  old_par <- graphics::par(no.readonly = TRUE)
+  # To please R CMD check
+  n <- distn <- fun_args <- pdf_or_cdf <- show_dens <- n_add <- rfun <-
+    discrete_distn <- top_range <- dfun <- xlab <- top_leg_pos <- arrow <-
+    distn_mean <- distn_sd <- bottom_p_vec <- bottom_leg_pos <- leg_cex <- NULL
+  panel <- within(panel, {
     # Don't add the rug in the top plot if n is large
     if (n > 1000) {
       show_rug <- FALSE
     } else {
       show_rug <- TRUE
     }
-    par(mfrow = c(2, 1), oma = c(0, 0, 0, 0), mar = c(4, 4, 2, 2) + 0.1)
+    graphics::par(mfrow = c(2, 1), oma = c(0, 0, 0, 0),
+                  mar = c(4, 4, 2, 2) + 0.1)
     # Do the simulation (if required)
     if (distn == "hypergeometric") {
       sim_list <- c(list(nn = n), fun_args)
@@ -483,10 +507,10 @@ cltmovie_plot <- function(panel) {
       mean_y <- apply(temp, 2, mean)
       # Extract the last dataset and the last mean (for drawing the arrow)
       y <- temp[, n_add]
-      assign("old_y", y, envir = envir)
+      old_y <- y
       rm(temp)
       last_y <- mean_y[n_add]
-      assign("save_last_y", last_y, envir = envir)
+      save_last_y <- last_y
     } else {
       mean_y <- NULL
       y <- old_y
@@ -497,7 +521,6 @@ cltmovie_plot <- function(panel) {
     } else {
       sample_means <- c(sample_means, mean_y)
     }
-    assign("sample_means", sample_means, envir = envir)
     #
     n_x_axis <- 501
     # Top plot --------
@@ -560,12 +583,13 @@ cltmovie_plot <- function(panel) {
       graphics::axis(1, line = 0.5)
       graphics::title(main = paste(the_distn, ",  n = ", n))
       graphics::legend(top_leg_pos, legend = expression(f(x)),
-                       col = 1, lwd = 2, lty = 2, box.lty = 0)
+                       col = 1, lwd = 2, lty = 2, box.lty = 0, cex = leg_cex)
       if (show_rug) {
         graphics::rug(y, line = 0.5, ticksize = 0.05)
       }
     } else {
-      ep <- 0.2
+      ep <- (my_xlim[2] - my_xlim[1] ) / 50
+      my_xlim[2] <- my_xlim[2] + ep
       y_vals <- table(y)
       x_vals <- as.numeric(names(y_vals))
       graphics::matplot(x_vals, y_vals, type = "n", xlab = xlab,
@@ -577,9 +601,9 @@ cltmovie_plot <- function(panel) {
       graphics::axis(1, line = 0.5)
       graphics::title(main = paste(the_distn, ",  n = ", n))
       graphics::legend(top_leg_pos, legend = c("sample", "P(X = x)"),
-                       col = 1, lwd = 2, lty = 1:2, box.lty = 0)
+                       col = 1, lwd = 2, lty = 1:2, box.lty = 0, cex = leg_cex)
     }
-    u_t <- par("usr")
+    u_t <- graphics::par("usr")
     if (arrow) {
       graphics::segments(last_y, u_t[3], last_y, -10, col = "red", xpd = TRUE,
                          lwd = 2, lty = 2)
@@ -653,19 +677,20 @@ cltmovie_plot <- function(panel) {
     if (pdf_or_cdf == "pdf") {
       if (show_dens) {
         graphics::legend(bottom_leg_pos, legend = my_leg_2, col = 1:2, lwd = 2,
-                         lty = 2, box.lty = 0)
+                         lty = 2, box.lty = 0, cex = leg_cex)
       }
     } else {
       if (show_dens) {
-        graphics::legend(bottom_leg_pos,
+        graphics::legend("topleft",
                        legend = c(my_leg_2, "empirical cdf"),
                        col = c(1, 8), lwd = 2, lty = c(2, -1),
-                       pch = c(-1, 16), box.lty = 0)
+                       pch = c(-1, 16), box.lty = 0, cex = leg_cex)
       } else {
-        graphics::legend(bottom_leg_pos,
+        graphics::legend("topleft",
                          legend = c(my_leg_2, "empirical cdf"),
                          col = c(0, 8), lwd = 2, lty = c(2, -1),
-                         pch = c(-1, 16), box.lty = 0, text.col = c(0, 1))
+                         pch = c(-1, 16), box.lty = 0, text.col = c(0, 1),
+                         cex = leg_cex)
       }
     }
     top_ratio <- (last_y - u_t[1]) / (u_t[2] - u_t[1])
@@ -677,10 +702,9 @@ cltmovie_plot <- function(panel) {
                        xpd = TRUE, code = 2)
     }
     old_n <- n
-    assign("old_n", old_n, envir = envir)
-    assign("old_pdf_or_cdf", pdf_or_cdf, envir = envir)
-    assign("old_show_dens", show_dens, envir = envir)
-    graphics::par(old_par)
+    old_pdf_or_cdf <- pdf_or_cdf
+    old_show_dens <- show_dens
   })
-  return(invisible(panel))
+  graphics::par(old_par)
+  return(panel)
 }
