@@ -118,6 +118,11 @@ continuous <- function(distn, var_range = NULL, params = list(),
                        param_step = list(), param_range = list(),
                        p_vec = NULL, smallest = 0.01, plot_par = list(),
                        panel_plot = TRUE, hscale = NA, vscale = hscale, ...) {
+  if (!tcltk::is.tclObj(tcltk::tclRequire("BWidget"))) {
+    message("Package BWidget was not found.")
+    message("Please see the smovie README file for information.")
+    return()
+  }
   # To add another distribution
   # 1. misc.R: add code to set_fun_args(), parameter_range(), parameter_step(),
   #            set_p_vec()
@@ -258,11 +263,15 @@ continuous <- function(distn, var_range = NULL, params = list(),
   if (is.null(p_vec)) {
     p_vec <- set_p_vec(distn)
   }
+  # Set a unique panel name to enable saving of objects to the correct panel
+  now_time <- strsplit(substr(date(), 12, 19), ":")[[1]]
+  now_time <- paste(now_time[1], now_time[2], now_time[3], sep = "")
+  my_panelname <- paste("continuous_", now_time, sep = "")
   # A list of arguments to pass to the plotting function via rp.control()
   pass_args <- fun_args
   names(pass_args) <- par_names
   my_title <- paste("pdf and cdf of the", root_name, "distribution")
-  for_rp_control <- c(list(title = my_title,
+  for_rp_control <- c(list(title = my_title, panelname = my_panelname,
                            dfun = dfun, pfun = pfun, qfun = qfun,
                            distn = distn, fun_args = fun_args, n_pars = n_pars,
                            par_names = par_names, pdf_or_cdf = pdf_or_cdf,
@@ -307,13 +316,15 @@ continuous <- function(distn, var_range = NULL, params = list(),
   rpanel::rp.radiogroup(panel= continuous_panel, pdf_or_cdf, c("pdf", "cdf"),
                         title = "pdf or cdf",
                         action = action)
-  rpanel::rp.do(continuous_panel, action = action)
+  if (!panel_plot) {
+    rpanel::rp.do(continuous_panel, action = action)
+  }
   return(invisible())
 }
 
 plot_continuous <- function(panel) {
+  old_par <- graphics::par(no.readonly = TRUE)
   with(panel, {
-    old_par <- graphics::par(no.readonly = TRUE)
     # Put the parameter values in a named list
     new_fun_args <- list()
     for (i in 1:n_pars) {
@@ -376,37 +387,40 @@ plot_continuous <- function(panel) {
     var_range <- seq(from = var_range[1], to = var_range[2], len = 1001)
     my_col <- "black"
     fun_args <- new_fun_args
-    my_xlab <- ifelse(!is.null(plot_par$xlab), plot_par$xlab, "x")
+    # Set default graphical parameters, except when supplied in plot_par
+    plot_par$xlab <- ifelse(!is.null(plot_par$xlab), plot_par$xlab, "x")
     my_ylab <- ifelse(pdf_or_cdf == "pdf", "density", "probability")
-    my_ylab <- ifelse(!is.null(plot_par$ylab), plot_par$ylab, my_ylab)
-    my_bty <- ifelse(!is.null(plot_par$bty), plot_par$bty, "l")
+    plot_par$ylab <- ifelse(!is.null(plot_par$ylab), plot_par$ylab, my_ylab)
+    plot_par$bty <- ifelse(!is.null(plot_par$bty), plot_par$bty, "l")
     if (pdf_or_cdf == "pdf") {
       probs <- do.call(dfun, c(list(x = var_range), fun_args))
       cond <- is.finite(probs)
       probs <- probs[cond]
       var_range <- var_range[cond]
       my_ylim <- c(0, max(probs))
-      my_main <- ifelse(!is.null(plot_par$main), plot_par$main,
-                        paste("pdf of the", the_distn, "distribution"))
+      if (is.null(plot_par$ylim)) {
+        plot_par$ylim <- my_ylim
+      }
+      plot_par$main <- ifelse(!is.null(plot_par$main), plot_par$main,
+                              paste("pdf of the", the_distn, "distribution"))
     } else {
       probs <- do.call(pfun, c(list(q = var_range), fun_args))
       cond <- is.finite(probs)
       probs <- probs[cond]
       var_range <- var_range[cond]
       my_ylim <- c(0, 1)
-      my_main <- ifelse(!is.null(plot_par$main), plot_par$main,
-                        paste("cdf of the", the_distn, "distribution"))
+      if (is.null(plot_par$ylim)) {
+        plot_par$ylim <- my_ylim
+      }
+      plot_par$main <- ifelse(!is.null(plot_par$main), plot_par$main,
+                              paste("cdf of the", the_distn, "distribution"))
     }
-    if (!is.null(plot_par$col)) {
-      my_col <- plot_par$col
-    } else {
-      my_col <- "black"
+    if (is.null(plot_par$col)) {
+      plot_par$col <- "black"
     }
-    for_plot <- list(x = var_range, y = probs, type = "l", xlab = my_xlab,
-                     ylab = my_ylab, col = my_col, bty = my_bty,
-                     main = my_main, ylim = my_ylim)
+    for_plot <- c(list(x = var_range, y = probs, type = "l"), plot_par)
     do.call(graphics::plot, for_plot)
-    graphics::par(old_par)
   })
+  graphics::par(old_par)
   return(panel)
 }
